<?php

/**
 * Déclaration de la classe CommandLine\Argument\Value\ValueArgument.
 */

namespace jrosset\CommandLine\Argument\Value;

use InvalidArgumentException;
use jrosset\CommandLine\Argument\ArgumentAbstract;

/**
 * Classe abstraite pour les paramètres de type "valeur"
 *
 * @package CommandLine\Argument\Value
 */
abstract class ValueAbstract extends ArgumentAbstract implements IArgumentValue {
    /**
     * @var int Le nombre minimum d'occurence.
     */
    protected int $occurMin = 1;
    /**
     * @var int|null Le nombre maximum d'occurence.
     */
    protected ?int $occurMax = 1;

    /**
     * Crée un nouvel argument.
     *
     * @param string      $name        Le nom de l'argument
     * @param string|null $description La description de l'argument
     * @param boolean     $optional    Argument optionel ?
     */
    protected function __construct (string $name, ?string $description, bool $optional) {
        parent::__construct($name, $description);

        $this->setOptional($optional);
    }

    /**
     * Est-ce que l'argument est facultatif ?
     *
     * @return bool Est-ce que l'argument est facultatif ?
     *
     * @noinspection PhpUnused
     */
    public function isOptional (): bool {
        return $this->getOccurMin() === 0;
    }
    /**
     * Définit si l'argument est facultatif ou non.
     *
     * @param bool $optional Argument facultatif ?
     *
     * @return $this
     */
    public function setOptional (bool $optional): self {
        $this->setOccurMin($optional ? 0 : 1);
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getOccurMin (): int {
        return $this->occurMin;
    }
    /**
     * Définit le nombre minimum d'occurence.
     *
     * @param int $occurMin Le nombre minimum
     *
     * @return $this
     * @see $occurMin
     */
    public function setOccurMin (int $occurMin): self {
        if (!is_numeric($occurMin)) {
            throw new InvalidArgumentException('Le nombre minimum d\'ocurrence n\'est pas un entier');
        }

        if (floor($occurMin) != $occurMin) {
            throw new InvalidArgumentException('Le nombre minimum d\'ocurrence n\'est pas un entier');
        }

        $int = (int)$occurMin;
        if ($int < 0) {
            throw new InvalidArgumentException('Le nombre minimum d\'ocurrence n\'est pas un entier positif');
        }

        $this->occurMin = $int;
        return $this;
    }

    /**
     * @inheritDoc
     */
    public function getOccurMax (): int {
        return $this->occurMax;
    }
    /**
     * Définit le nombre mawimum d'occurence.
     *
     * @param int|null $occurMax Le nombre maximum
     *
     * @return $this
     * @see $occurMax
     */
    public function setOccurMax (?int $occurMax): self {
        if (!is_null($occurMax)) {
            if (!is_numeric($occurMax)) {
                throw new InvalidArgumentException('Le nombre maximum d\'ocurrence n\'est pas un entier');
            }

            if (floor($occurMax) != $occurMax) {
                throw new InvalidArgumentException('Le nombre maximum d\'ocurrence n\'est pas un entier');
            }

            $occurMax = (int)$occurMax;
            if ($occurMax <= 0) {
                throw new InvalidArgumentException('Le nombre maximum d\'ocurrence n\'est pas un entier strictement positif');
            }
        }

        $this->occurMax = $occurMax;
        return $this;
    }
}